#ifndef _WINC_DRAWING
#define _WINC_DRAWING

#include <WXP/Date.h>
#include <WXP/Domain.h>
#include <WXP/Plot.h>

namespace WXP {
   /** \class Drawing Drawing.h "../include/Drawing.h"
    *  \brief This is a virtual class defining drawing routines
    *
    *  \author Dan Vietor
    *  \ingroup Plot
    */
   
   class Drawing {
      public:
      virtual ~Drawing() {}
      /**********************************************************************/
      /** Initializes object
       */
      virtual int init() = 0;
      /**********************************************************************/
      /** A method for getting a default domain from the data.  This is handy 
       * for data types that are for a specific domain like NIDS data.
       * \retval domain	Default domain
       */
      virtual int getDomain( Domain &domain ) = 0;
      /**********************************************************************/
      /** A method for setting basic file parameters
      */
      virtual int setFile() = 0;
      /**********************************************************************/
      /** A method for setting input file type
      * \param name	Name convention to use
      */
      virtual int setInput( const char *type ) = 0;
      /**********************************************************************/
      /** A method for setting input file name convention
      * \param name	Name convention to use
      */
      virtual int setNameConv( const char *name ) = 0;
      /**********************************************************************/
      /** A method for setting the default file for plotting
      */
      virtual int setFilename() = 0;
      /**********************************************************************/
      /** A method for setting the default file for plotting
      * \param file	Filename to use
      */
      virtual int setFilename( const char *file ) = 0;
      /**********************************************************************/
      /** Sets the filename to use based on a specific date
      * \param date	Date to use
      */
      virtual int setFilename( Date &date ) = 0;
      /**********************************************************************/
      /** Sets the filename to use based on a specific date and parameters
      * \param date	Date to use
      * \param param	Parameters to use for offset to date
      */
      virtual int setFilename( Date &date, const char *param ) = 0;
      /**********************************************************************/
      /** Sets specific forecast time to plot
      * \param fore	Forecast time
      */
      virtual int setForeTime( const char *fore ) = 0;
      /**********************************************************************/
      /** Sets specific parameters for the plot given a string
      * \param str	String specifying parameters
      */
      virtual int setParam( const char *str ) = 0;
      /**********************************************************************/
      /** Gets the current date of the data in the file
      * \retval date	Date information 
      */
      virtual int getDate( Date &date ) = 0;
      /**********************************************************************/
      /** Gets the available loop types
      * \return Loop type
      */
      virtual int getLoopType() = 0;
      /**********************************************************************/
      /** Gets the initial date for a loop
      * \retval date	Date information 
      */
      virtual int getInitDate( Date &date ) = 0;
      /**********************************************************************/
      /** Gets the offset time between frames in loop
      * \retval date	Date information 
      */
      virtual int getDateOffset() = 0;
      /**********************************************************************/
      /** Gets the possible number of frames in a loop
      * \return Number of frames
      */
      virtual int getNumFrames() = 0;
      /**********************************************************************/
      /** Tells whether an object should be used to sync other to it
      * \return Number of frames
      */
      virtual int getSync() = 0;
     /**********************************************************************/
     /** Method for plotting the actual data
     * \param plot	The plot class specifying window and parameters 
     *  for plotting
     */
     virtual int draw( Plot &plot ) = 0;

     /**********************************************************************/
     /** Method for printing class info
     */
     virtual int print() = 0;

     static const int LOOP_NONE = 0; /**< No loop */
     static const int LOOP_DATE = 1; /**< Loop on date */
     static const int LOOP_FORE = 2; /**< Loop on forecast time */
   };
}   
#endif
